# Advent of Code 12/8/2022, puzzle 1 -- counting visible trees

# In my perfect world I'd have thought of a way to characterize the mask generating code
#  in a generic way that'd allow code reuse instead of have four substantially repeating
#  copies of the same small routine.  But ah well.

import sys
import copy

infile = "8_1_input.txt"

data = open(infile)

forest = list(data)
trees = []
for i in range(len(forest)):
	# this list comprehension stuff seems nice...
	newline = [int(x) for x in forest[i].strip()]
	trees.append(newline)

width = len(trees[0])
height = len(trees)
print("width: ", width, " height: ", height)

# idea: create four directional masks that mark every cell as visible or shadowed
#  from that direction; then count visible trees by checking each cell in all four
#  masks for visibility; any tree visible in at least one direction counts as visible.

leftmask = copy.deepcopy(trees)
rightmask = copy.deepcopy(trees)
for i in range(height):
	lhigh = -1
	for j in range(width):
		t = leftmask[i][j]
		if t > lhigh:
			lhigh = t
			leftmask[i][j] = True
		else:
			leftmask[i][j] = False

	rhigh = -1
	for j in reversed(range(width)):
		t = rightmask[i][j]
		if t > rhigh:
			rhigh = t
			rightmask[i][j] = True
		else:
			rightmask[i][j] = False

# gotta transit vertically this time
topmask = copy.deepcopy(trees)
botmask = copy.deepcopy(trees)
for j in range(width):
	thigh = -1
	for i in range(height):
		t = topmask[i][j] 
		if t > thigh:
			thigh = t
			topmask[i][j] = True
		else:
			topmask[i][j] = False

	bhigh = -1
	for i in reversed(range(height)):
		t = botmask[i][j] 
		if t > bhigh:
			bhigh = t
			botmask[i][j] = True
		else:
			botmask[i][j] = False

#print(*botmask, sep='\n')

# And finally lets actually check those four masks for each cell
visiblecount = 0
for i in range(width):
	for j in range(height):
		if leftmask[i][j] or rightmask[i][j] or topmask[i][j] or botmask[i][j]:
			visiblecount += 1

print("Total visible trees: ", visiblecount)

