# doing some hatched grids

import numpy as np
import matplotlib.patches as mpatches
from matplotlib.collections import PatchCollection
import matplotlib.pyplot as plt

import random
import math
import copy


width = 20
height = 20
step = 10

squares = []


# generate a checkerboard of squares as a 2D array
def getsquaregrid(w=8,h=5,len=10):
    squareg = [None] * w
    for i in range(w):
        squareg[i] = [None] * h
        for j in range(h):
            x1 = i*len
            y1 = j*len
            x2 = (i+1)*len
            y2 = (j+1)*len
            newsq = [ [x1,y1], [x1,y2], [x2, y2], [x2, y1] ]
            squareg[i][j] = newsq
    return squareg


# hatch a polygon's interior with chaotic lines
def hackpoly(p):
    hatches = []
    edges = p
    numhatch = int(random.random()*50 + 5)

    for i in range(numhatch):
        [A,B] = random.sample(edges,2)
        p1 = getlinebisect(A[0], A[1], random.random())
        p2 = getlinebisect(B[0], B[1], random.random())
        hatches.append([p1, p2])

    return hatches


# chaotic hatching in a continuous polyline
def circuitpolyline(p, numhatch=10):
    hatches = []
    points = p
    cycle = len(points)
 #   numhatch = int(random.random()*25 + 3)
  
    # initial line
    [A,B] = [ [p[0], p[1]], [p[1],p[2]] ]
    p1 = getlinebisect(A[0], A[1], random.random())
    p2 = getlinebisect(B[0], B[1], random.random())
    hatches.append([p1, p2])

    for i in range(1,numhatch):
        #B = edges[(i+1)%cycle]
        B = [p[i%cycle], p[(i+1)%cycle]] 
        p1 = hatches[i-1][1]
        p2 = getlinebisect(B[0], B[1], random.random())
        hatches.append([p1, p2])

    hatches.append([hatches[-1][1], hatches[0][0]])
    
    return hatches




# buggy circuitpoly draft that is actually kinda cool tho
def laserpoly(p):
    hatches = []
    edges = p
    cycle = len(edges)
    numhatch = int(random.random()*50 + 5)
  
    # initial line
    [A,B] = [ edges[0], edges[1] ]
    p1 = getlinebisect(A[0], A[1], random.random())
    p2 = getlinebisect(B[0], B[1], random.random())
    hatches.append([p1, p2])

    for i in range(1,numhatch):
        B = edges[(i+1)%cycle]
        p1 = hatches[i-1][0]
        p2 = getlinebisect(B[0], B[1], random.random())
        hatches.append([p1, p2])

    return hatches

# nicked from stack overflow, thank you random SO person for your service
def find_intersection( p0, p1, p2, p3 ) :

    s10_x = p1[0] - p0[0]
    s10_y = p1[1] - p0[1]
    s32_x = p3[0] - p2[0]
    s32_y = p3[1] - p2[1]
    denom = s10_x * s32_y - s32_x * s10_y
    if denom == 0 : return None # collinear

    denom_is_positive = denom > 0

    s02_x = p0[0] - p2[0]
    s02_y = p0[1] - p2[1]
    s_numer = s10_x * s02_y - s10_y * s02_x
    if (s_numer < 0) == denom_is_positive : return None # no collision

    t_numer = s32_x * s02_y - s32_y * s02_x
    if (t_numer < 0) == denom_is_positive : return None # no collision
    if (s_numer > denom) == denom_is_positive or (t_numer > denom) == denom_is_positive : return None # no collision

    # collision detected
    t = t_numer / denom
    intersection_point = [ p0[0] + (t * s10_x), p0[1] + (t * s10_y) ]

    return intersection_point

# hatch the interior of convex polygon poly by cropping some lines to fit
# angle is in radians, convert from degrees before passing
def crophatch(poly, angle=0.001, step=1):
    inhatch = []  # the uncropped hatch lines
    outhatch = []  # the cropped hatch we're producing

    # let's get a tight bounding box around this polygon

    xmin = poly[0][0]
    ymin = poly[0][1]
    xmax = 0
    ymax = 0
   
    for p in poly:
        if(p[0] < xmin): xmin = p[0]
        if(p[0] > xmax): xmax = p[0]
        if(p[1] < ymin): ymin = p[1]
        if(p[1] > ymax): ymax = p[1]


    # calculate the spans of these bounds, dy and dx, and expand them by a tiny amount to help make sure bounding
    # box intersections happen even with parallel edges on the source polyline
    dx = xmax - xmin
    dy = ymax - ymin
    xmin -= 0.00001
    ymin -= 0.00001
    xmax += 0.00001
    ymax += 0.00001
    
    # now let's generate some hash lines within those bounds
    #step = 0.5
    #slope = -1 # slope of our basic hatch line, use e.g. 999999 instead of dividing by 0 plz
    # four kinds of slope to handle in this hacky iterative line drawing way, handle each seperately

    #step = random.random()*2 + 0.25
    #slope = random.random()*20 - 10


    # let's do some trigonometry i guess
#    angle = angle/180 * math.pi
    #step = 1
    slope = math.tan(angle)
    # really ought to test for div/0 here...
    stepx = step/abs(math.sin(angle)) 
    stepy = step/abs(math.cos(angle)) 

#    print("angle, slope, stepx, stepy: ", angle, ", ", slope, ", ", stepx, ", ", stepy)


    if(slope >= 1):
        x = xmin - dx
        while x < xmax:
            inhatch.append([ [x,ymin], [x+(dy/slope),ymax] ])
            x += stepx
    elif(slope >= 0):
        y = ymin - dy
        while y < ymax:
            inhatch.append([ [xmin,y], [xmax,y+(dx*slope)] ])
            y += stepy
    elif(slope >= -1):
        y = ymax + dy
        while y > ymin:
            inhatch.append([ [xmin,y], [xmax,y+(dx*slope)] ])
            y -= stepy
    else: 
        x = xmax + dx
        while x > xmin:
            inhatch.append([ [x,ymin], [x+(dy/slope),ymax] ])
            x -= stepx


    # now the doozy: for each hatch line, try to intersect with each poly line
    #  if we get two intersections, we should crop it top and bottom
    for l in inhatch:
        out = []
        p1 = l[0]
        p2 = l[1]
#        print("p1 p2", p1, " ", p2)
        for i in range(len(poly)):
            p3 = poly[i]
            if(i == len(poly)-1): p4 = poly[0]
            else: p4 = poly[i+1]
#            print("p3 p4", p3, " ", p4)
            newp = find_intersection(p1,p2,p3,p4)
            if(newp != None):
                out.append(newp)
        # there might be 2, 4, 6...intersections in valid hatch line vs. a non-convex poly; handling this
        # correctly requires sorting the segments of the carved up hatch line in asc. or desc. order along
        # the line, e.g. by x or y coord (edge case for cardinal lines I suppose)
#        print("out: ",out)
        if(len(out) > 1 and len(out) % 2 == 0):
            out.sort()
            for i in range(0,len(out),2):
                outhatch.append([out[i],out[i+1]])

    return outhatch




#given a start and an end coordinate and a midpoint position 0..1, return a new coord at that point on line AB
def getlinebisect(A,B,portion):
    x1 = A[0]
    y1 = A[1]
    x2 = B[0]
    y2 = B[1]
    
    dx = x2-x1
    dy = y2-y1

    newx = x1 + dx*portion
    newy = y1 + dy*portion

    return[newx,newy]

def copyshift(lines,xshift,yshift):
    lout = []
    for l in lines:
        lnew = copy.deepcopy(l)
        lnew[0][0] += xshift
        lnew[0][1] += yshift
        lnew[1][0] += xshift
        lnew[1][1] += yshift
        lout.append(lnew)
    return lout

def slant(lines,slantmag=2):
    for l in lines:
        mag = 1/(len(l))
        for p in l:
            p[0] += (random.random() - 0.5) * slantmag * mag
    return lines

def twist(lines):
    for l in lines:
        l[1][0] = width - l[1][0]
    return lines

def feather(lines,feathermag=2):
    for l in lines:
        for p in l:
            p[1] += (random.random() - 0.5) * feathermag
    return lines

def divide(lines, iterations):
    newl = []
    for k in range(iterations):
        for i in range(len(lines)):
            newl.append( [lines[i][0]] )
            for j in range(len(lines[i])-1): 
                p1 = lines[i][j]
                p2 = lines[i][j+1]
                mid = [ (p1[0] + p2[0]) / 2,  ( p1[1] + p2[1]) / 2  ]
                newl[i].append(mid)
                newl[i].append(p2)
        
        lines = newl
        newl = []
    return lines


#######

fig, ax = plt.subplots(figsize=(11, 8.5))

patches = []
#squares = getsquares()
squareg = getsquaregrid(width,height,step)

#count = 5
#for sq in squares:
#    count += 1

''' pool of light
cx = random.random()*18 
cy = random.random()*18
a = 0

for i in range(len(squareg)):
    for j in range(len(squareg[i])):
        dx = i-cx
        dy = j-cy
        a = math.atan(dy/dx) + math.pi/2
        lines = crophatch(squareg[i][j],a,4/math.sqrt(dx**2 + dy**2))
'''

for i in range(len(squareg)):
    for j in range(len(squareg[i])):
        #lines = circuitpolyline(squareg[i][j])

        #patches.append(mpatches.Polygon(squareg[i][j],closed=True,fill=None,color="black"))

 #       val = 3
#        val = int(random.random()*15)+3

#        val = 0
#        if( (i ^ j) % 9 == 0): val = 10
#

        lines = circuitpolyline(squareg[i][j],int((i+j)**1.1 + 2))
 #       lines = circuitpolyline(squareg[i][j], val)
        a = 0




    #    lines = divide(lines,8)
    #    lines = slant(lines)
    #    lines = feather(lines)
        for l in lines:
            patches.append(mpatches.Polygon(l,closed=False,fill=None,color="black"))





plt.grid(False)
plt.axis('off')
ax.set_aspect('equal')

x_bounds = [-5, width*step + 5]
y_bounds = [-5, height*step + 5]

ax.set_xlim(x_bounds)
ax.set_ylim(y_bounds)

collection = PatchCollection(patches, match_original=True)
ax.add_collection(collection)

plt.savefig('grid_hatch.svg', bbox_inches = 'tight', pad_inches = 0)
plt.show()


